;**********************************
;  This file is part of
; "Simple SEGA Master System BIOS" 
;  By David Senabre Albujer.
;
;  Sprite and pad control
;
; Assembler: WLA_DX
;**********************************
    
    ;===============================
    ; Rutinas de control de sprites
    ; Manejo del pad
    ; Carga del menu 
    ;===============================

.section "Actualizar Sprites" free
    ;--------------------------------------------------------------
    ; Copiar Sprites a la VRAM
    ;--------------------------------------------------------------
RefreshSprites:
push   hl
push   bc
push   af
    ; Acceder al comienzo de la tabla de sprites en VRAM
    ld    hl,SpriteTableAddress
    call  VRAMToHL

    ld    a,(BallYh)           ; Cargar la posicion Y del sprite
    out   ($be),a             ; Escribirlo en la VRAM
    ;  Saltar ahora la siguientes 127 posiciones
    ;  dedicadas a la posicion Y de otros posibles sprites
    ld    hl,SpriteTableAddress+128
    call  VRAMToHL
    ld    a,(BallXh)           ;  Transferir la posicion X
    out   ($be),a
    ld    a,(BallTile)         ;  Transferir el indice de tile
    out   ($be),a
pop    af
pop    bc
pop    hl
ret

    ;---------------------------------------
    ; Aplicar movimiento a sprites
    ;---------------------------------------
MoveSprites:
push af
push bc
    ld a,0

    ld a,(Up)          ;comprobar si pulsamos arriba
    bit 0,a
    jp z, +            ;si no pulsado, saltar
    ld a,(BallSYl)     ;si esta pulsado
    sub 8              ;restar 8 a la velcidad en Y
    ld (BallSYl),a
    ld a,(BallSYh)     ;y si ha habido acarreo
    sbc a,0            ;retarselo al byte alto
    ld (BallSYh),a
    +:                 ;ver ahora si pulsamos abajo
    ld a,(Down)        ;y lo mismo que arriba
    bit 0,a            ; solo que ahora sumamos 8
    jp z, ++
    ld a,(BallSYl)
    add a,8
    ld (BallSYl),a
    ld a,(BallSYh)
    adc a,0
    ld (BallSYh),a

    ;  Repetir lo mismo que arriba pero para la direccion
    ;  izquierda y derecha con la velocidad en X
    ++:                ;
    ld a,(Left)
    bit 0,a
    jp z, +++
    ld a,(BallSXl)
    sub 8
    ld (BallSXl),a
    ld a,(BallSXh)
    sbc a,0
    ld (BallSXh),a

    +++:
    ld a,(Right)
    bit 0,a
    jp z, ++++

    ld a,(BallSXl)
    add a,8
    ld (BallSXl),a
    ld a,(BallSXh)
    adc a,0
    ld (BallSXh),a

    ; Ahora vamos a actualizar la posicion del sprite
    ; sumando las velocidades a su respectivas posiciones
    ; primero pasamos los 2 bytes de la velocidad en Y
    ; a los registros B y C
    ++++:
    ld a,(BallYl)
    ld b,a
    ld a,(BallYh)
    ld c,a

    ; Ahora sumarlo a la posicion, cargando el byte bajo en A
    ; y sumando el byte bajo de velocidad
    ; y luego al byte alto, el byte alto de velocidad mas
    ; el posible acarreo de la suma anterior
    ld a,(BallSYl)
    add a,b
    ld (BallYl),a
    ld a,(BallSYh)
    adc a,c
    ld (BallYh),a

    ; lo mismo con la coordenada X
    ld a,(BallXl)
    ld b,a
    ld a,(BallXh)
    ld c,a
    ld a,(BallSXl)
    add a,b
    ld (BallXl),a
    ld a,(BallSXh)
    adc a,c
    ld (BallXh),a

pop bc
pop af
ret

.ends

.section "Leer pad" free
    ;---------------------------------------
    ; leer estado de pad1
    ;---------------------------------------
/*    7: Joypad 2 Down
      6: Joypad 2 Up
      5: Joypad 1 Fire B
      4: Joypad 1 Fire A
      3: Joypad 1 Right
      2: Joypad 1 Left
      1: Joypad 1 Down
      0: Joypad 1 Up   */
GetPad1:
  push  af
  push  bc
  ld    a,0    ;resetear las variables del pad
  ld    (Button1),a
  ld    (Button2),a
  ld    (Up),a
  ld    (Down),a
  ld    (Left),a
  ld    (Right),a
  in    a,($DC);leer el estado del pad
  cpl          ;complementar sus bits
  ld    b,a    ;y guardarlo en B
  ld    a,0    ;A sera un registro auxiliar

  rr    b      ;rotar B a traves del flag carry
  rl    a      ;e introducir por la izquierda en A
  ld    (Up),a ;ahi tenemos el estado "Arriba"
  ld    a,0    ;reseteamos A
               ;para recibir el siguiente boton
  rr    b      ;volvemos a rotar al carry
  rl    a      ;y a introducir por la izquierda en A
  ld    (Down),a  ;y asi sucesivamente...
  ld    a,0

  rr    b
  rl    a
  ld    (Left),a
  ld    a,0

  rr    b
  rl    a
  ld    (Right),a
  ld    a,0

  rr    b
  jp    nc,+
  ;boton 1 pulsado
  ld    a,1
  ld    (Button1),a
  +:
  rr    b
  jp    nc,++
  ld    a,1
  ld    (Button2),a
  ++:
  pop   bc
  pop   af
ret

.ends

.section "Cargar menu" free
;---------------------------------------
CargarMenu:
;---------------------------------------
    ld       bc,0
    ld       a,0
    call     ScreenOff    ;apagar la imagen para acceder a VRAM
    call     ClearScreen  ;borrar la pantalla de texto
    ld       bc,Titulo
    ld       hl,64*4
    ld       d,255
    call     PrintScreenExt
    ld       bc,SistemaString
    ld       hl,64*12
    ld       d,255
    call     PrintScreenExt
    ld       bc,CartString
    ld       hl,64*17
    ld       d,255
    call     PrintScreenExt
    call     PrintDomesticMode
    call     PrintTVMode
    call     CheckCart
    call     PrintCart
    call     ScreenOn             ;volver a encender la imagen
    ld       a,1
    ld       (Mode),a               ;pasar a modo=1 (menu)
    ld       a,0                    ;inicializar variables
    ld       (ScrollY),a            ;resetear scroll a 0
    out      ($bf),a               ;registro de ScrollY
    ld       a,$89
    out      ($bf),a
  ret


.ends

